time.init <- Sys.time()

####### Loading file with data  ##########################
# data file that is loaded needs to have four columns: 1) number of the dataset; 2) mean age estimates; 3) minimum age range estimate; 4) maximum age range estimate
# Address of the data file should be corrected to its location on the computer
datalist <- read.csv("###file location###", header=FALSE)


####### Alpha/significance value and the number of iterations for resampling####################################
alpha <- 0.05

iteration <- 5000

acheulean <- c(###outlier date###,###outlier date###)

######### Creating table for results ############################
SolSmith.results <- matrix(0,4,5)
#for.plots <- matrix(0,iteration,max(datalist[,1]))
SolSmith.results[,1] <- c("Forward 5", "Forward 10", "Reverse 5", "Reverse 10")


######## SolSmith calculation function ###############################
SolSmith.test <-
  function(sights,y){
    k <- length(sights)
    sights <- sort(sights, decreasing = TRUE)
    calcsum <- 0
    for (i in 1:(k-1)) {
      calcsum <- calcsum + (i+1)*(sights[i]-sights[i+1])
    }
    sk <- (y-sights[1])/(y-sights[1]+calcsum) 
    res <- (1-sk)^(k-1)
    return(res)	
  }


####### applying SolSmith to each of the datasets ###########
for (i in 1:max(datalist[,1])) {
  # data file is split into separate datasets based on the first column
  dataset <- datalist[which(datalist[,1]==i),2:4]
  # record timing is adapted for SolSmith by expressing them as time since the earliest record in the dataset

  mean.record <- max(dataset[,1]) - dataset[,1] + 1
  if(i <= 4) {
    acheulean.rec <- max(dataset[,1]) - acheulean + 1
    # SolSmith function is applied to the data & results are included in the matrix with results
    SolSmith.results[i,2] <- SolSmith.test(mean.record,acheulean.rec[1])
    SolSmith.results[i,3] <- SolSmith.test(mean.record,acheulean.rec[2])    
  } else {
    acheulean.rec <- max(dataset[,1])
    SolSmith.results[i,2] <- SolSmith.test(mean.record,acheulean.rec)
  }
    

  
  # vectors are established where results from resampling will be stored
  CI.A.list <- CI.B.list <- {}
  loop <- 1
#   start of the resampling loop
   while(loop <= iteration) {
    #resampling from a normal distribution based on range estimates, with sd equal to a half difference between mean and min/max estimates
    sd.low <- abs(rnorm(length(dataset[,1]), m=0, sd=((dataset[,1]-dataset[,3])/2)))
    sd.high <- abs(rnorm(length(dataset[,1]), m=0, sd=((dataset[,2]-dataset[,1])/2)))
    sampling <- matrix(0,length(dataset[,1]),2)
    sampling[,1] <- dataset[,1] - sd.low
    sampling[,2] <- dataset[,1] + sd.high
    rr <- rbinom(length(dataset[,1]),1,0.5)+1
    dd <- {}
    for(ii in c(1:length(dataset[,1]))){
      dd <- c(dd,sampling[ii,rr[ii]])
    }
    mean.record.iter <- max(dd) - dd + 1
    if(i <= 4) {
      acheulean.rec.iter <- max(dd) - acheulean + 1
      CI.A.list <- c(CI.A.list, SolSmith.test(mean.record.iter,acheulean.rec.iter[1]))
      CI.B.list <- c(CI.B.list, SolSmith.test(mean.record.iter,acheulean.rec.iter[2]))
    } else {
      acheulean.rec.iter <- max(dd)
      CI.A.list <- c(CI.A.list, SolSmith.test(mean.record.iter,acheulean.rec.iter))
    }
    loop <- loop + 1
  }
  #mean values are taken from vectors with results are included in the matrix with results
  if(i <= 4) {
    SolSmith.results[i,4] <- mean(CI.A.list)
    SolSmith.results[i,5] <- mean(CI.B.list)
  } else {
    SolSmith.results[i,4] <- mean(CI.A.list)
  }
#  for.plots[,i] <- TE.list
  print(i)
}


####### show SolSmith results ##################################################
print(SolSmith.results)
write.csv(SolSmith.results, file="###file loction###")
#write.csv(for.plots, file="###file location###")
Sys.time() - time.init
